@@ -1,5 +1,5 @@
 /* internal.h --- Internal header file for GSS.
- * Copyright (C) 2003, 2004, 2005, 2006, 2007  Simon Josefsson
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of the Generic Security Service (GSS).
  *
@@ -35,8 +35,8 @@
 #include <time.h>
 
 /* Get specification. */
-#include "api.h"
-#include "ext.h"
+#include <gss/api.h>
+#include <gss/ext.h>
 
 /* Get xmalloc etc. */
 #include "xalloc.h"
