@@ -1,5 +1,5 @@
 /* krb5/msg.c --- Implementation of Kerberos 5 GSS-API Per-Message functions.
- * Copyright (C) 2003, 2004, 2005, 2006, 2007  Simon Josefsson
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Simon Josefsson
  *
  * This file is part of the Generic Security Service (GSS).
  *
@@ -61,6 +61,7 @@ gss_krb5_wrap (OM_uint32 * minor_status,
   _gss_krb5_ctx_t k5 = context_handle->krb5;
   size_t padlength;
   gss_buffer_desc data;
+  char *p;
   size_t tmplen;
   int rc;
 
@@ -87,7 +88,7 @@ gss_krb5_wrap (OM_uint32 * minor_status,
 	 */
 	padlength = 8 - input_message_buffer->length % 8;
 	data.length = 4 * 8 + input_message_buffer->length + padlength;
-	data.value = xmalloc (data.length);
+	p = xmalloc (data.length);
 
 	/* XXX encrypt data iff confidential option chosen */
 
@@ -101,17 +102,17 @@ gss_krb5_wrap (OM_uint32 * minor_status,
 	  return GSS_S_FAILURE;
 
 	/* Compute checksum over header, confounder, input string, and pad */
-	memcpy (data.value, header, 8);
-	memcpy (data.value + 8, confounder, 8);
-	memcpy (data.value + 16, input_message_buffer->value,
+	memcpy (p, header, 8);
+	memcpy (p + 8, confounder, 8);
+	memcpy (p + 16, input_message_buffer->value,
 		input_message_buffer->length);
-	memset (data.value + 16 + input_message_buffer->length,
-		padlength, padlength);
+	memset (p + 16 + input_message_buffer->length,
+		(int) padlength, padlength);
 
 	rc = shishi_checksum (k5->sh,
 			      k5->key,
 			      0, SHISHI_RSA_MD5_DES_GSS,
-			      data.value,
+			      p,
 			      16 + input_message_buffer->length + padlength,
 			      &cksum, &tmplen);
 	if (rc != SHISHI_OK || tmplen != 8)
@@ -142,16 +143,18 @@ gss_krb5_wrap (OM_uint32 * minor_status,
 	  return GSS_S_FAILURE;
 
 	/* put things in place */
-	memcpy (data.value, header, 8);
-	memcpy (data.value + 8, eseqno, 8);
+	memcpy (p, header, 8);
+	memcpy (p + 8, eseqno, 8);
 	free (eseqno);
-	memcpy (data.value + 16, cksum, 8);
+	memcpy (p + 16, cksum, 8);
 	free (cksum);
-	memcpy (data.value + 24, confounder, 8);
-	memcpy (data.value + 32, input_message_buffer->value,
+	memcpy (p + 24, confounder, 8);
+	memcpy (p + 32, input_message_buffer->value,
 		input_message_buffer->length);
-	memset (data.value + 32 + input_message_buffer->length,
-		padlength, padlength);
+	memset (p + 32 + input_message_buffer->length,
+		(int) padlength, padlength);
+
+	data.value = p;
 
 	rc = gss_encapsulate_token (&data, GSS_KRB5, output_message_buffer);
 	if (!rc)
@@ -168,67 +171,70 @@ gss_krb5_wrap (OM_uint32 * minor_status,
 	char *tmp;
 
 	padlength = 8 - input_message_buffer->length % 8;
-	data.length =
-	  8 + 8 + 20 + 8 + input_message_buffer->length + padlength;
-	data.value = xmalloc (data.length);
+	data.length = 8 + 8 + 20 + 8 + input_message_buffer->length
+	  + padlength;
+	p = xmalloc (data.length);
 
 	/* XXX encrypt data iff confidential option chosen */
 
 	/* Compute checksum over header, confounder, input string, and pad */
 
-	memcpy (data.value, TOK_WRAP, 2);	/* TOK_ID: Wrap */
-	memcpy (data.value + 2, "\x04\x00", 2);	/* SGN_ALG: 3DES */
-	memcpy (data.value + 4, "\xFF\xFF", 2);	/* SEAL_ALG: none */
-	memcpy (data.value + 6, "\xFF\xFF", 2);	/* filler */
-	rc = shishi_randomize (k5->sh, 0, data.value + 8, 8);
+	memcpy (p, TOK_WRAP, 2);	/* TOK_ID: Wrap */
+	memcpy (p + 2, "\x04\x00", 2);	/* SGN_ALG: 3DES */
+	memcpy (p + 4, "\xFF\xFF", 2);	/* SEAL_ALG: none */
+	memcpy (p + 6, "\xFF\xFF", 2);	/* filler */
+	rc = shishi_randomize (k5->sh, 0, p + 8, 8);
 	if (rc != SHISHI_OK)
 	  return GSS_S_FAILURE;
-	memcpy (data.value + 16, input_message_buffer->value,
+	memcpy (p + 16, input_message_buffer->value,
 		input_message_buffer->length);
-	memset (data.value + 16 + input_message_buffer->length,
-		padlength, padlength);
+	memset (p + 16 + input_message_buffer->length,
+		(int) padlength, padlength);
 
 	rc = shishi_checksum (k5->sh,
 			      k5->key,
 			      SHISHI_KEYUSAGE_GSS_R2,
-			      SHISHI_HMAC_SHA1_DES3_KD, data.value,
+			      SHISHI_HMAC_SHA1_DES3_KD, p,
 			      16 + input_message_buffer->length + padlength,
 			      &tmp, &tmplen);
 	if (rc != SHISHI_OK || tmplen != 20)
 	  return GSS_S_FAILURE;
 
-	memcpy (data.value + 16, tmp, tmplen);
-	memcpy (data.value + 36, data.value + 8, 8);
+	memcpy (p + 16, tmp, tmplen);
+	memcpy (p + 36, p + 8, 8);
 
 	/* seq_nr */
 	if (k5->acceptor)
 	  {
-	    ((char *) data.value + 8)[0] = k5->acceptseqnr & 0xFF;
-	    ((char *) data.value + 8)[1] = k5->acceptseqnr >> 8 & 0xFF;
-	    ((char *) data.value + 8)[2] = k5->acceptseqnr >> 16 & 0xFF;
-	    ((char *) data.value + 8)[3] = k5->acceptseqnr >> 24 & 0xFF;
-	    memset (data.value + 8 + 4, 0xFF, 4);
+	    (p + 8)[0] = k5->acceptseqnr & 0xFF;
+	    (p + 8)[1] = k5->acceptseqnr >> 8 & 0xFF;
+	    (p + 8)[2] = k5->acceptseqnr >> 16 & 0xFF;
+	    (p + 8)[3] = k5->acceptseqnr >> 24 & 0xFF;
+	    memset (p + 8 + 4, 0xFF, 4);
 	  }
 	else
 	  {
-	    ((char *) data.value + 8)[0] = k5->initseqnr & 0xFF;
-	    ((char *) data.value + 8)[1] = k5->initseqnr >> 8 & 0xFF;
-	    ((char *) data.value + 8)[2] = k5->initseqnr >> 16 & 0xFF;
-	    ((char *) data.value + 8)[3] = k5->initseqnr >> 24 & 0xFF;
-	    memset (data.value + 8 + 4, 0, 4);
+	    (p + 8)[0] = k5->initseqnr & 0xFF;
+	    (p + 8)[1] = k5->initseqnr >> 8 & 0xFF;
+	    (p + 8)[2] = k5->initseqnr >> 16 & 0xFF;
+	    (p + 8)[3] = k5->initseqnr >> 24 & 0xFF;
+	    memset (p + 8 + 4, 0, 4);
 	  }
 
-	rc = shishi_encrypt_iv_etype (k5->sh, k5->key, 0, SHISHI_DES3_CBC_NONE, data.value + 16, 8,	/* cksum */
-				      data.value + 8, 8, &tmp, &tmplen);
+	rc = shishi_encrypt_iv_etype (k5->sh, k5->key, 0, SHISHI_DES3_CBC_NONE,
+				      p + 16, 8,	/* cksum */
+				      p + 8, 8, &tmp, &tmplen);
 	if (rc != SHISHI_OK || tmplen != 8)
 	  return GSS_S_FAILURE;
 
-	memcpy (data.value + 8, tmp, tmplen);
+	memcpy (p + 8, tmp, tmplen);
 	free (tmp);
-	memcpy (data.value + 8 + 8 + 20 + 8, input_message_buffer->value,
+	memcpy (p + 8 + 8 + 20 + 8, input_message_buffer->value,
 		input_message_buffer->length);
-	memset (data.value + 8 + 8 + 20 + 8 + input_message_buffer->length,
-		padlength, padlength);
+	memset (p + 8 + 8 + 20 + 8 + input_message_buffer->length,
+		(int) padlength, padlength);
+
+	data.value = p;
 
 	rc = gss_encapsulate_token (&data, GSS_KRB5, output_message_buffer);
 	if (!rc)
@@ -291,14 +297,14 @@ gss_krb5_unwrap (OM_uint32 * minor_status,
     case 0:			/* DES-MD5 */
       {
 	size_t padlen;
-	unsigned char *pt;
+	char *pt;
 	char header[8];
 	char encseqno[8];
 	char seqno[8];
 	char cksum[8];
 	char confounder[8];
 	char *tmp;
-	int seqnr;
+	uint32_t seqnr;
 	size_t outlen, i;
 
 	/* Typical data:
@@ -351,7 +357,7 @@ gss_krb5_unwrap (OM_uint32 * minor_status,
 	if (padlen > 8)
 	  return GSS_S_BAD_MIC;
 	for (i = 1; i <= padlen; i++)
-	  if (((char *) data)[datalen - i] != padlen)
+	  if (((char *) data)[datalen - i] != (int)padlen)
 	    return GSS_S_BAD_MIC;
 
 	/* Write header and confounder next to data */
@@ -387,6 +393,7 @@ gss_krb5_unwrap (OM_uint32 * minor_status,
 	char *t;
 	char cksum[20];
 	size_t outlen, i;
+	uint32_t seqnr;
 
 	if (datalen < 8 + 8 + 20 + 8 + 8)
 	  return GSS_S_BAD_MIC;
@@ -409,7 +416,8 @@ gss_krb5_unwrap (OM_uint32 * minor_status,
 	if (memcmp (p + 4, k5->acceptor ? "\x00\x00\x00\x00" :
 		    "\xFF\xFF\xFF\xFF", 4) != 0)
 	  return GSS_S_BAD_MIC;
-	if (C2I (p) != (k5->acceptor ? k5->initseqnr : k5->acceptseqnr))
+	seqnr = C2I (p);
+	if (seqnr != (k5->acceptor ? k5->initseqnr : k5->acceptseqnr))
 	  return GSS_S_BAD_MIC;
 
 	if (k5->acceptor)
@@ -422,7 +430,7 @@ gss_krb5_unwrap (OM_uint32 * minor_status,
 	if (padlen > 8)
 	  return GSS_S_BAD_MIC;
 	for (i = 1; i <= padlen; i++)
-	  if (((char *) data)[datalen - i] != padlen)
+	  if (((char *) data)[datalen - i] != (int) padlen)
 	    return GSS_S_BAD_MIC;
 
 	/* Write header next to confounder */
